import itertools
import vptree
from geographiclib.geodesic import Geodesic
import cartopy.io.shapereader as shpreader
from shapely.geometry import LinearRing


def geodesic_distance(lon_lat_1, lon_lat_2):
    """Calculate geodesic distance between a pair of points in km.

    Based on https://stackoverflow.com/a/45480555
    """
    return (
        Geodesic.WGS84.Inverse(lon_lat_1[1], lon_lat_1[0], lon_lat_2[1], lon_lat_2[0])[
            "s12"
        ]
        / 1000
    )


def coastline_coords(lat_range, lon_range, resolution="10m"):
    """Get list of coastline coordinates from Natural Earth Data coastline sections
    that intersect with the lat_range and lon_range.
    """
    x = shpreader.natural_earth(resolution="10m", category="physical", name="coastline")
    cl = shpreader.Reader(x)
    coastlines = cl.records()
    my_bounds = LinearRing(
        [
            (lon_range[0], lat_range[0]),
            (lon_range[0], lat_range[1]),
            (lon_range[1], lat_range[1]),
            (lon_range[1], lat_range[0]),
        ]
    )
    in_bounds = [cl for cl in coastlines if cl.geometry.intersects(my_bounds)]
    coords = list(itertools.chain(*[ib.geometry.coords for ib in in_bounds]))
    return coords


def build_vptree(lat_range, lon_range, **kwargs):
    """Build vantage-point tree from Natural Earth Data coastline sections that
    intersect with the lat_range and lon_range.

    Use vpt.get_nearest_neighbor((lon, lat)) to find geodesic distance of nearest point
    on the coastline to (lon, lat) in km.

    Based on https://stackoverflow.com/a/45480555
    """
    coords = coastline_coords(lat_range, lon_range, **kwargs)
    vpt = vptree.VPTree(coords, geodesic_distance)
    return vpt



import pandas as pd
from . import Dbs


def concat(objs, logfile=None, **kwargs):
    """Concatenate imported .dbs files, before determining sessions."""
    obj = pd.concat(objs, **kwargs)
    obj.reset_index(drop=True, inplace=True)
    obj = Dbs(obj)
    obj.logfile = logfile
    return obj



import pandas as pd


class Dbs(pd.DataFrame):
    logfile = None
    sessions = None

    from .get import (
        get_logfile_index,
        get_sample_blanks,
        get_session_blanks,
        get_blank_corrections,
        get_density,
        get_standard_calibrations,
        get_session_calibrations,
        calibrate_dic,
    )
    from .plot import plot_blanks, plot_session_blanks, plot_k_dic, plot_dic_offset

    def subset(self, condition, batch_col="dic_cell_id"):
        """Create a subset of this dbs based on the logical condition."""
        lf = self.logfile
        ss = self.sessions
        dbs_subset = Dbs(self[condition])
        dbs_subset.logfile = lf
        dbs_subset.sessions = ss.loc[self[batch_col].unique()]
        return dbs_subset



import koolstof as ks, numpy as np, pandas as pd

# Import the VINDTA logfile
fpath = "tests/data/"
logfile = ks.read_logfile(
    fpath + "logfile_20200407.bak",
    methods=["3C standard", "3C standardRWS"],
)

# Import a .dbs file for part of that logfile
dbs = ks.read_dbs(fpath + "2018_Aug_RWS_CO2.dbs", logfile=logfile)

# Add metadata
dbs["dic_certified"] = np.where(dbs.station == 666, 2029.19, np.nan)
dbs["salinity"] = np.where(dbs.station == 666, 33.434, 35)

# Do QC
dbs["blank_good"] = True
dbs.loc[dbs.bottle == "WALCRN2_2018005578", "blank_good"] = False

# Do processing
dbs.calibrate_dic()
sub = dbs.subset(dbs.dic_cell_id == "C_Aug13-18_1008")


def dbs_assertions(dbs):
    assert isinstance(dbs, pd.DataFrame)
    assert isinstance(dbs, ks.Dbs)
    assert hasattr(dbs, "sessions")
    assert isinstance(dbs.sessions, pd.DataFrame)
    assert hasattr(dbs, "logfile")
    assert isinstance(dbs.logfile, pd.DataFrame)


def test_import_and_calibration():
    assert "dic" in dbs
    dbs_assertions(dbs)


# def test_plots():
#     dbs.plot_blanks()
#     dbs.plot_k_dic()
#     dbs.plot_dic_offset()


def test_subset():
    dbs_assertions(sub)
    sub.plot_dic_offset()


# test_import_and_calibration()
# test_plots()
# test_subset()
